/*******************************************************************************
 *
 * Copyright (C) 2022 NETINT Technologies
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 ******************************************************************************/

/*!*****************************************************************************
*  \file   ni_quadra_filter_api.h
*
*  \brief  NETINT filters API header file
*
*******************************************************************************/

#pragma once
#ifdef __cplusplus
extern "C" {
#endif
// #include "ni_defs.h"
// #include "ni_rsrc_api.h"
// #include "ni_util.h"
// #include "ni_device_api.h"
// #include "ni_av_codec.h"
// #include "ni_bitstream.h"
#define LIB_API

#ifdef _WIN32
#include <windows.h>
#include <process.h>
#include <malloc.h>
#elif __linux__ || __APPLE__
#if __linux__
#include <linux/types.h>
#endif
#include <sys/time.h>
#include <pthread.h>
#include <errno.h>
#include <semaphore.h>
#endif

#include <assert.h>
#include <ctype.h>
#include <inttypes.h>
#include <signal.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "ni_macro_defs_quadra.h"

#ifndef __va_list__
typedef __gnuc_va_list va_list;
#endif /* not __va_list__ */

#define NI_QUADRA_BEST_MODEL_LOAD_STR "bestmodelload"
#define NI_QUADRA_BEST_REAL_LOAD_STR "bestload"

// if you change it, you need change ni_device_api.h
// NI_MAX_KEEP_ALIVE_TIMEOUT NI_MIN_KEEP_ALIVE_TIMEOUT NI_DEFAULT_KEEP_ALIVE_TIMEOUT
#define NI_QUADRA_MAX_KEEP_ALIVE_TIMEOUT 100
#define NI_QUADRA_MIN_KEEP_ALIVE_TIMEOUT 1
#define NI_QUADRA_DEFAULT_KEEP_ALIVE_TIMEOUT 3

// if you change it, you need change ni_device_api.h MAX_NUM_FRAMEPOOL_HWAVFRAME
#define MAX_QUADRA_NUM_FRAMEPOOL_HWAVFRAME 128


// if change it, you need change ni_def.h NI_MAX_NUM_DATA_POINTERS
#define NI_QUADRA_MAX_NUM_DATA_POINTERS 4

// if change it, you need change ni_device_api.h NI_MAX_NUM_AUX_DATA_PER_FRAME
#define NI_QUADRA_MAX_NUM_AUX_DATA_PER_FRAME 16

// if change it, you need change ni_def.h NI_MAX_NUM_OF_DECODER_OUTPUTS
#define NI_QUADRA_MAX_NUM_OF_DECODER_OUTPUTS 3

// if change it, you need change ni_def.h NI_MAX_PPU_PARAM_EXPR_CHAR
#define NI_QUADRA_MAX_PPU_PARAM_EXPR_CHAR 20

// if change it, you need change ni_device_api.h NI_MAX_FIFO_CAPACITY
#define NI_QUADRA_MAX_FIFO_CAPACITY 120

// if change it, you need change ni_device_api.h NI_NOPTS_VALUE
#define NI_QUADRA_NOPTS_VALUE ((int64_t)UINT64_C(0x8000000000000000))

/*Values below used for VPU resolution range checking*/
// if change it, you need change ni_device_api.h
// NI_MAX_WIDTH NI_MIN_WIDTH NI_MAX_HEIGHT NI_MIN_HEIGHT
#define NI_QUADRA_MAX_WIDTH 8192
#define NI_QUADRA_MIN_WIDTH 144
#define NI_QUADRA_MAX_HEIGHT 8192
#define NI_QUADRA_MIN_HEIGHT 128

#define NIALIGN(x, a) (((x) + (a)-1) & ~((a)-1))
#define NIMAX(a, b) ((a) > (b) ? (a) : (b))
#define NIMIN(a, b) ((a) > (b) ? (b) : (a))

// if change it, you need change ni_device_api.h
#define NI_QUADRA_SCALER_FLAG_IO                                               \
    0x0001 /* 0 = source frame, 1 = destination frame */
#define NI_QUADRA_SCALER_FLAG_PC                                               \
    0x0002 /* 0 = single allocation, 1 = create pool */
#define NI_QUADRA_SCALER_FLAG_PA                                               \
    0x0004 /* 0 = straight alpha, 1 = premultiplied alpha */
#define NI_QUADRA_SCALER_FLAG_P2                                               \
    0x0008 /* 0 = normal allocation, 1 = P2P allocation */
#define NI_QUADRA_SCALER_FLAG_FCE                                              \
    0x0010 /* 0 = no fill color, 1 = fill color enabled */
#define NI_QUADRA_SCALER_FLAG_CS                                               \
    0x0020 /* 0 = BT.709, 1 = BT.2020 */
#define NI_QUADRA_SCALER_FLAG_CMP                                              \
    0x0800 /* 0 = no compress, 1 = compress*/

#define NI_QUADRA_AI_FLAG_IO 0x0001 /* 0 = source frame, 1 = destination frame */
#define NI_QUADRA_AI_FLAG_PC 0x0002 /* 0 = single allocation, 1 = create pool */

// if change it, you need change ni_device_api.h
// NI_MAX_RESOLUTION_WIDTH NI_MAX_RESOLUTION_HEIGHT
#define NI_QUADRA_MAX_RESOLUTION_WIDTH 8192
#define NI_QUADRA_MAX_RESOLUTION_HEIGHT 8192

#define NI_QUADRA_MIN_RESOLUTION_WIDTH_SCALER 32
#define NI_QUADRA_MIN_RESOLUTION_HEIGHT_SCALER 32

// if change it, you need change ni_device_api.h NI_MAX_SUPPORT_DRAWBOX_NUM
#define NI_QUADRA_MAX_SUPPORT_DRAWBOX_NUM 5
#define NI_QUADRA_MAX_SUPPORT_WATERMARK_NUM 6

#define NI_QUADRA_MAX_DEVICE_CNT      128

// Assume that each buffer is the same size
typedef struct _ni_fifo_buffer_t
{
    pthread_mutex_t mutex;
    uint32_t number_of_buffers;   // total number of buffers
    uint32_t buffer_size;         // the size of every buffer
    uint32_t number_of_buffers_used;
    uint8_t *buffer;
    uint32_t rndx, wndx;
} ni_fifo_buffer_t;

typedef enum
{
    NI_QUADRA_LOG_INVALID = -1,   // invalid selection
    NI_QUADRA_LOG_NONE = 0,       // display no logging
    NI_QUADRA_LOG_FATAL = 1,   // log messages immediately prior to program exit
    NI_QUADRA_LOG_ERROR = 2,   // error messages
    NI_QUADRA_LOG_INFO = 3,    // info and warning messages
    NI_QUADRA_LOG_DEBUG = 4,   // very verbose messages about program execution
    NI_QUADRA_LOG_TRACE = 5   // most verbose messages (eg. function enter/exit, NVMe
                              // transactions, read/write polling retries)
} ni_quadra_log_level_t;

typedef enum _ni_quadra_pixel_planar_format
{
    NI_QUADRA_PIXEL_PLANAR_FORMAT_SEMIPLANAR = 0,
    NI_QUADRA_PIXEL_PLANAR_FORMAT_PLANAR = 1,
    NI_QUADRA_PIXEL_PLANAR_FORMAT_TILED4X4 = 2,
    NI_QUADRA_PIXEL_PLANAR_MAX
} ni_quadra_pixel_planar_format;
// if change it, you need change ni_def.h ni_scaler_opcode_t
typedef enum _ni_quadra_scaler_opcode
{
    NI_QUADRA_SCALER_OPCODE_SCALE = 0,
    NI_QUADRA_SCALER_OPCODE_CROP = 1,
    NI_QUADRA_SCALER_OPCODE_FLIP = 1,
    NI_QUADRA_SCALER_OPCODE_PAD = 2,
    NI_QUADRA_SCALER_OPCODE_OVERLAY = 3,
    NI_QUADRA_SCALER_OPCODE_STACK = 4,
    NI_QUADRA_SCALER_OPCODE_ROTATE = 5,
    NI_QUADRA_SCALER_OPCODE_DRAWBOX = 6,
    NI_QUADRA_SCALER_OPCODE_IPOVLY = 7,
    NI_QUADRA_SCALER_OPCODE_WATERMARK = 8,
    NI_QUADRA_SCALER_OPCODE_DELOGO = 9,
    NI_QUADRA_SCALER_OPCODE_MERGE = 10
} ni_quadra_scaler_opcode_t;

// if change it, you need change ni_device_api.h ni_codec_format_t
typedef enum _ni_quadra_codec_format
{
    NI_QUADRA_CODEC_FORMAT_H264 = 0,
    NI_QUADRA_CODEC_FORMAT_H265 = 1,
    NI_QUADRA_CODEC_FORMAT_VP9 = 2,
    NI_QUADRA_CODEC_FORMAT_JPEG = 3,
    NI_QUADRA_CODEC_FORMAT_AV1 = 4
} ni_quadra_codec_format_t;

typedef enum
{
    NI_QUADRA_RET_SUCCESS = 0,
    NI_QUADRA_SEQ_CHANGE_DRAINING = -1024,
    NI_QUADRA_ERROR_VPU_RECOVERY = -1023,
    NI_QUADRA_ERROR_INVALID_SESSION = -1022,
    NI_QUADRA_EAGAIN = -1021,
    NI_QUADRA_ERROR = -1020,
    NI_QUADRA_PARAM_WARNING_DEPRECATED = -1019,
} ni_quadra_retcode_t;

// if change it, you need change ni_def.h ni_device_type_t
typedef enum
{
    NI_QUADRA_DEVICE_TYPE_MIN = -1,

    // xcoder instance types
    NI_QUADRA_DEVICE_TYPE_DECODER = 0,
    NI_QUADRA_DEVICE_TYPE_ENCODER = 1,
    NI_QUADRA_DEVICE_TYPE_SCALER = 2,
    NI_QUADRA_DEVICE_TYPE_AI = 3,
    NI_QUADRA_DEVICE_TYPE_XCODER_MAX = 4,

    // pseudo types
    NI_QUADRA_DEVICE_TYPE_UPLOAD =
        4,   // share instance with NI_DEVICE_TYPE_ENCODER

    NI_QUADRA_DEVICE_TYPE_MAX = 5,
} ni_quadra_device_type_t;

typedef enum
{
    NI_QUADRA_PIX_FMT_YUV420P = 0,      /* 8-bit YUV420 planar       */
    NI_QUADRA_PIX_FMT_YUV420P10LE = 1,  /* 10-bit YUV420 planar      */
    NI_QUADRA_PIX_FMT_NV12 = 2,         /* 8-bit YUV420 semi-planar  */
    NI_QUADRA_PIX_FMT_P010LE = 3,       /* 10-bit YUV420 semi-planar */
    NI_QUADRA_PIX_FMT_RGBA = 4,         /* 32-bit RGBA packed        */
    NI_QUADRA_PIX_FMT_BGRA = 5,         /* 32-bit BGRA packed        */
    NI_QUADRA_PIX_FMT_ARGB = 6,         /* 32-bit ARGB packed        */
    NI_QUADRA_PIX_FMT_ABGR = 7,         /* 32-bit ABGR packed        */
    NI_QUADRA_PIX_FMT_BGR0 = 8,         /* 32-bit RGB packed         */
    NI_QUADRA_PIX_FMT_BGRP = 9,         /* 24-bit RGB packed         */
    NI_QUADRA_PIX_FMT_NV16 = 10,        /* 8-bit YUV422 semi-planar  */
    NI_QUADRA_PIX_FMT_YUYV422 = 11,     /* 8-bit YUV422              */
    NI_QUADRA_PIX_FMT_UYVY422 = 12,     /* 8-bit YUV422              */
    NI_QUADRA_PIX_FMT_8_TILED4X4 = 13,  /* 8 bit tiled4x4            */
    NI_QUADRA_PIX_FMT_10_TILED4X4 = 14, /* 10 bit tiled4x4           */
    NI_QUADRA_PIX_FMT_NONE = 15,        /* invalid format            */
    NI_QUADRA_PIX_FMT_YUV420P10BE = 16, /* 10-bit YUV420 planar      */
    NI_QUADRA_PIX_FMT_YUVJ420P = 17,
} ni_quadra_pix_fmt_t;

// if change it, you need change ni_device_api.h ni_frame_config_t
typedef struct _ni_quadra_frame_config
{
    uint16_t picture_width;
    uint16_t picture_height;
    uint16_t picture_format;
    uint16_t options;
    uint16_t rectangle_width;
    uint16_t rectangle_height;
    int16_t rectangle_x;
    int16_t rectangle_y;
    uint32_t rgba_color;
    uint16_t frame_index;
    uint16_t session_id;
    uint8_t output_index;
    uint8_t orientation;   // 0 <= n <= 3, (n * 90°) clockwise rotation
} ni_quadra_frame_config_t;

typedef struct _ni_quadra_network_layer_params_t
{
    uint32_t num_of_dims; /* The number of dimensions specified in *sizes */
    uint32_t sizes[6];    /* The pointer to an array of dimension */
    int32_t
        data_format; /* Data format for the tensor, see ni_ai_buffer_format_e */
    int32_t
        quant_format; /* Quantized format see ni_ai_buffer_quantize_format_e */
    union
    {
        struct
        {
            int32_t
                fixed_point_pos; /* Specifies the fixed point position when the input element type is int16, if 0 calculations are performed in integer math */
        } dfp;

        struct
        {
            float scale;       /* Scale value for the quantized value */
            int32_t zeroPoint; /* A 32 bit integer, in range [0, 255] */
        } affine;
    } quant_data; /* The union of quantization information */
    /* The type of this buffer memory. */
    uint32_t memory_type;
} ni_quadra_network_layer_params_t;

typedef struct _ni_quadra_network_layer_info
{
    ni_quadra_network_layer_params_t *in_param;
    ni_quadra_network_layer_params_t *out_param;
} ni_quadra_network_layer_info_t;

typedef struct _ni_quadra_network_layer_offset
{
    int32_t offset;
} ni_quadra_network_layer_offset_t;

typedef struct _ni_quadra_network_data
{
    uint32_t input_num;
    uint32_t output_num;
    ni_quadra_network_layer_info_t linfo;
    ni_quadra_network_layer_offset_t
        *inset; /* point to each input layer start offset from p_frame */
    ni_quadra_network_layer_offset_t
        *outset; /* point to each output layer start offset from p_packet */
} ni_quadra_network_data_t;

// if change this structure, you need to change ni_device_api.h ni_frameclone_desc_t
typedef struct _ni_quadra_frameclone_desc
{
    uint16_t ui16SrcIdx;
    uint16_t ui16DstIdx;
    uint32_t ui32Offset;
    uint32_t ui32Size;
    uint32_t reserved;
} ni_quadra_frameclone_desc_t;

/*!*
* \brief This is an enumeration for hw actions
*/
typedef enum ni_quadra_codec_hw_actions
{
    NI_QUADRA_CODEC_HW_NONE = 0,
    NI_QUADRA_CODEC_HW_ENABLE = (1 << 0),
    NI_QUADRA_CODEC_HW_DOWNLOAD = (1 << 1),
    NI_QUADRA_CODEC_HW_UPLOAD = (1 << 2),
    NI_QUADRA_CODEC_HW_RSVD = (1 << 3),
    NI_QUADRA_CODEC_HW_PAYLOAD_OFFSET = 4
} ni_quadra_codec_hw_actions_t;

typedef struct _ni_quadra_filter_t
{
    int32_t device_handle;
    int32_t blk_io_handle;
    /*! Device Type, Either NI_DEVICE_TYPE_DECODER or NI_DEVICE_TYPE_ENCODER */
    uint32_t device_type;
    /*! Codec ID */
    int hw_id;
    int hw_action;
    uint32_t scaler_operation;
    /* from the command line, which resource allocation method we use */
    void *api_ctx;
    void *params;
    void *dst_frame;
    /* for drawtext filter use */
    void *dl_frame;
    void *txt_frame;
    void *dst_pkt;

    int initialized;
    int session_opened;
    int keep_alive_timeout; /* keep alive timeout setting */
    bool is_p2p;
    int filterblit;
    int autoselect;
    double scaler_param_b;
    double scaler_param_c;
} ni_quadra_filter_t;

typedef struct _ni_quadra_split_context_t
{
    int enabled;
    int w[3];
    int h[3];
    int f[3];     //planar format
    int f8b[3];   //forced 8bit
                  //int crop_meta_data_rltb[3][4]; //crop rectangle
} ni_quadra_split_context_t;
// if change it, you need change ni_device_api.h niFrameSurface1_t
typedef struct _niquadraFrameSurface1
{
    uint16_t ui16FrameIdx;      //frame location on device
    uint16_t ui16session_ID;    //for instance tracking
    uint16_t ui16width;         // width on device
    uint16_t ui16height;        // height on device
    uint32_t ui32nodeAddress;   //currently not in use, formerly offset
    int32_t device_handle;      //handle to access device
    int8_t bit_depth;           //1 == 8bit per pixel, 2 == 10bit
    int8_t encoding_type;       //0 = semiplanar, 1 = planar, 2 = tiled4x4
    int8_t output_idx;          // 0-2 for decoder output index
    int8_t src_cpu;             // frame origin location
    int32_t dma_buf_fd;         // P2P dma buffer file descriptor
} niquadraFrameSurface1_t;

typedef struct NIFramesContext
{
    niquadraFrameSurface1_t *surfaces_internal;
    int nb_surfaces_used;
    niquadraFrameSurface1_t **surface_ptrs;
    void *api_ctx;               //for down/uploading frames
    void *src_session_io_data;   // for upload frame to be sent up
    ni_quadra_split_context_t split_ctx;
    void *dst_frame;
    int32_t suspended_device_handle;
    int uploader_device_id;   //same one passed to libxcoder session open
} NIFramesContext;
/**
* This struct is allocated as AVHWDeviceContext.hwctx
*/
typedef struct AVNIDeviceContext
{
    int uploader_ID;
    int32_t uploader_handle;

    int32_t cards[128];
} AVNIDeviceContext;

// if change it, you need change ni_device_api.h ni_scaler_drawbox_params_t
typedef struct _ni_quadra_scaler_drawbox_params_t
{
    uint32_t start_x;
    uint32_t start_y;
    uint32_t end_x;
    uint32_t end_y;
    uint32_t rgba_c;
} ni_quadra_scaler_drawbox_params_t;

typedef struct _ni_quadra_scaler_watermark_params_t{
    uint32_t ui32StartX;
    uint32_t ui32StartY;
    uint32_t ui32Width;
    uint32_t ui32Height;
    uint32_t ui32Valid;
} ni_quadra_scaler_watermark_params_t;

// if change it, you need change ni_device_api.h ni_scaler_multi_drawbox_params_t
typedef struct _ni_quadra_scaler_multi_drawbox_params_t
{
    ni_quadra_scaler_drawbox_params_t
        multi_drawbox_params[NI_QUADRA_MAX_SUPPORT_DRAWBOX_NUM];
} ni_quadra_scaler_multi_drawbox_params_t;

typedef struct _ni_quadra_scaler_multi_watermark_params_t
{
    ni_quadra_scaler_watermark_params_t 
        multi_watermark_params[NI_QUADRA_MAX_SUPPORT_WATERMARK_NUM];
} ni_quadra_scaler_multi_watermark_params_t;

typedef struct _ni_quadra_card_info
{
  int card_idx;
  int load;
  int model_load;
  int task_num;
  int max_task_num;
  int shared_mem_usage;
  int firmware_load;
} ni_quadra_card_info_t;

typedef struct _ni_quadra_hw_device_info
{
  int available_card_num;
  int device_type_num;
  int consider_mem;
  int *device_type;/*! 0 for decoder,1 for encoder*/
  ni_quadra_card_info_t **card_info; 
  int card_current_card;
  int err_code;
} ni_quadra_hw_device_info_t;

typedef struct _ni_quadra_hw_device_info_encoder_param
{
    uint32_t fps;/*! encoder fps*/
    uint32_t h;/*! height*/
    uint32_t w;/*! width*/
    uint32_t code_format;/*! 1 for h264,2 for h265,3 for av1,4 for JPEG*/
    uint32_t ui8enableRdoQuant;
    uint32_t rdoLevel;
    uint32_t lookaheadDepth;/*! lookaheadDepth [0:40]*/
    uint32_t bit_8_10;/*! 8 for 8 bit,10 for 10 bit*/
    int uploader;/*1 for uploader,0 for not uploader*/
    int rgba;/*1 for rgba,0 for not*/
}ni_quadra_hw_device_info_encoder_param_t;

typedef struct _ni_quadra_hw_device_info_decoder_param
{
    uint32_t fps;/*! decoder fps*/
    uint32_t h;/*! height*/
    uint32_t w;/*! width*/
    uint32_t bit_8_10;/*! 8 for 8 bit,10 for 10 bit*/
    int rgba;/*!decoder for rgba? 1 for rgba,0 for not*/
    int hw_frame;/*if out=hw,1 for out = hw_frame,0 for not*/
}ni_quadra_hw_device_info_decoder_param_t;

typedef struct _ni_quadra_hw_device_info_scaler_param
{
    uint32_t h;/*!output height*/
    uint32_t w;/*!output width*/
    uint32_t bit_8_10;/*! 8 for 8 bit,10 for 10 bit*/
    int rgba;/*!output for rgba? 1 for rgba,0 for not*/
}ni_quadra_hw_device_info_scaler_param_t;

typedef struct _ni_quadra_hw_device_info_ai_param
{
    uint32_t h;/*!output height*/
    uint32_t w;/*!output width*/
    uint32_t bit_8_10;/*! 8 for 8 bit,10 for 10 bit*/
    int rgba;/*!output for rgba? 1 for rgba,0 for not*/
}ni_quadra_hw_device_info_ai_param_t;

typedef struct _ni_quadra_hw_device_info_threshold_param
{
    int device_type;/*! 0 for decoder,1 for encoder,2 for scaler*/
    int load_threshold;
    int task_num_threshold;
}ni_quadra_hw_device_info_threshold_param_t;

typedef struct _ni_quadra_hw_device_info_coder_param
{
    int hw_mode;
    ni_quadra_hw_device_info_encoder_param_t *encoder_param;
    ni_quadra_hw_device_info_decoder_param_t *decoder_param;
    ni_quadra_hw_device_info_scaler_param_t *scaler_param;
    ni_quadra_hw_device_info_ai_param_t *ai_param;
}ni_quadra_hw_device_info_coder_param_t;

typedef struct _ni_quadra_device_queue
{
    uint32_t xcoder_cnt[NI_QUADRA_DEVICE_TYPE_XCODER_MAX];
    int32_t xcoders[NI_QUADRA_DEVICE_TYPE_XCODER_MAX][NI_QUADRA_MAX_DEVICE_CNT];
} ni_quadra_device_queue_t;

typedef struct _ni_quadra_device_pool
{
  int32_t lock;
  ni_quadra_device_queue_t *p_device_queue;
} ni_quadra_device_pool_t;

/*!*****************************************************************************
 *  \brief  Init filter
 *
 *  \param[in/out] nifilter  Pointer ni_quadra_filter_t
 *
 *  \return On success
 *                           NI_RETCODE_SUCCESS
 *          On failure
 *                           NI_RETCODE_FAILURE
 * 
 *******************************************************************************/
LIB_API int ni_quadra_init_filters(ni_quadra_filter_t *nifilter);

/*!*****************************************************************************
 *  \brief  Alloc frame pool for filter
 *
 *  \param[in/out] nifilter   Pointer ni_quadra_filter_t
 *  \param[in] width          frame width
 *  \param[in] height         frame height
 *  \param[in] out_format     frame format
 * 
 *  \return On success
 *                           NI_RETCODE_SUCCESS
 *          On failure
 *                           NI_RETCODE_FAILURE
 * 
 *******************************************************************************/
LIB_API int ni_quadra_build_frame_pool(ni_quadra_filter_t *nifilter, int width,
                                       int height, int out_format,
                                       int pool_size);

/*!*****************************************************************************
*  \brief  Allocate memory for the hwDescriptor buffer based on provided
*          parameters taking into account pic size and extra data.
*
*  \param[in/out] nifilter   Pointer ni_quadra_filter_t
*
*  \param[in] video_width   Width of the video frame
*  \param[in] video_height  Height of the video frame
*  \param[in] extra_len     Extra data size (incl. meta data)
*
*  \return On success
*                          NI_RETCODE_SUCCESS
*          On failure
*                          NI_RETCODE_FAILURE
*                          NI_RETCODE_ERROR_MEM_ALOC
*****************************************************************************/
LIB_API int
ni_quadra_filter_frame_buffer_alloc_hwenc(ni_quadra_filter_t *nifilter,
                                          int video_width, int video_height,
                                          int extra_len);

/*!*****************************************************************************
 *  \brief  Allocate a frame on the device for 2D engine or AI engine
 *          to work on based on provided parameters
 *
 *  \param[in/out] nifilter Pointer ni_quadra_filter_t
 *  \param[in]  width       width, in pixels
 *  \param[in]  height      height, in pixels
 *  \param[in]  format      pixel format
 *  \param[in]  options     options bitmap flags, bit 0 (NI_SCALER_FLAG_IO) is
 *              0=input frame or 1=output frame. Bit 1 (NI_SCALER_FLAG_PC) is
 *              0=single allocation, 1=create pool. Bit 2 (NI_SCALER_FLAG_PA) is
 *              0=straight alpha, 1=premultiplied alpha
 *  \param[in]  rectangle_width     clipping rectangle width
 *  \param[in]  rectangle_height    clipping rectangle height
 *  \param[in]  rectangle_x         horizontal position of clipping rectangle
 *  \param[in]  rectangle_y         vertical position of clipping rectangle
 *  \param[in]  rgba_color          RGBA fill colour (for padding only)
 *  \param[in]  frame_index         input hwdesc index
 *  \param[in]  device_type         only NI_DEVICE_TYPE_SCALER
 *              and NI_DEVICE_TYPE_AI (only needs p_ctx and frame_index)
 *
 *  \return         NI_RETCODE_FAILURE
 *                  NI_RETCODE_ERROR_INVALID_SESSION
 *                  NI_RETCODE_ERROR_NVME_CMD_FAILED
 *                  NI_RETCODE_ERROR_MEM_ALOC
 ******************************************************************************/
LIB_API int ni_quadra_filter_device_alloc_frame(
    ni_quadra_filter_t *nifilter, int width, int height, int format,
    int options, int rectangle_width, int rectangle_height, int rectangle_x,
    int rectangle_y, int rgba_color, int frame_index, int type);

/*!*****************************************************************************
 *  \brief  Allocate a frame on the device and return the frame index
 *
 *  \param[in]  p_ctx  pointer to session context
 *  \param[in]  p_out_surface  pointer to output frame surface
 *  \param[in]  device_type    currently only NI_DEVICE_TYPE_AI
 *
 *  \return         NI_RETCODE_INVALID_PARAM
 *                  NI_RETCODE_ERROR_INVALID_SESSION
 *                  NI_RETCODE_ERROR_UNSUPPORTED_FW_VERSION
 *                  NI_RETCODE_ERROR_NVME_CMD_FAILED
 *                  NI_RETCODE_ERROR_MEM_ALOC
 ******************************************************************************/
LIB_API int ni_quadra_filter_device_alloc_dst_frame(ni_quadra_filter_t *nifilter,
                                            niquadraFrameSurface1_t *p_hwdesc,
                                            int type);

/*!*****************************************************************************
 *  \brief  Copy the data of src hwframe to dst hwframe
 *
 *  \param[in]  p_ctx  pointer to session context
 *  \param[in]  p_frameclone_desc  pointer to the frameclone descriptor
 *
 *  \return         NI_RETCODE_INVALID_PARAM
 *                  NI_RETCODE_ERROR_UNSUPPORTED_FW_VERSION
 *                  NI_RETCODE_ERROR_INVALID_SESSION
 *                  NI_RETCODE_ERROR_NVME_CMD_FAILED
 *                  NI_RETCODE_ERROR_MEM_ALOC
 ******************************************************************************/
LIB_API int ni_quadra_filter_device_clone_hwframe(
                    ni_quadra_filter_t *nifilter,
                    ni_quadra_frameclone_desc_t *p_frameclone_desc);

/*!*****************************************************************************
*  \brief  Read data from the device
*          If device_type is NI_DEVICE_TYPE_DECODER reads data hwdesc from
*          decoder
*          If device_type is NI_DEVICE_TYPE_SCALER reads data hwdesc from
*          scaler
*
*  \param[in/out] p_ctx    Pointer to void *
*                          ni_session_context_t struct
*  \param[in/out] p_data   Pointer to void *
*                          ni_session_data_io_t struct
*  \param[in] device_type  NI_DEVICE_TYPE_DECODER or NI_DEVICE_TYPE_SCALER
*                          If NI_DEVICE_TYPE_DECODER or NI_DEVICE_TYPE_SCALER is specified,
*                          hw descriptor info will be stored in p_data ni_frame
*  \return On success
*                          Total number of bytes read
*          On failure
*                          NI_RETCODE_FAILURE
*                          NI_RETCODE_ERROR_NVME_CMD_FAILED
*                          NI_RETCODE_ERROR_INVALID_SESSION
******************************************************************************/
LIB_API int
ni_quadra_device_session_read_hwdesc(void *p_ctx, void *p_data,
                                            int device_type);

/*!*****************************************************************************
 *  \brief  Read data from the device
 *          If device_type is NI_DEVICE_TYPE_DECODER reads data packet from
 *          decoder
 *          If device_type is NI_DEVICE_TYPE_ENCODER reads data frame from
 *          encoder
 *          If device_type is NI_DEVICE_TYPE_AI reads data frame from AI engine
 *
 *  \param[in/out] p_ctx    Pointer to void *
 *                          ni_session_context_t struct
 *  \param[in/out] p_data   Pointer to void *
 *                          ni_session_data_io_t struct
 *  \param[in] device_type  NI_DEVICE_TYPE_DECODER, NI_DEVICE_TYPE_ENCODER, or
 *                          NI_DEVICE_TYPE_SCALER
 *                          If NI_DEVICE_TYPE_DECODER is specified, data that
 *                          was read will be placed into ni_frame_t struct
 *                          inside the p_data pointer
 *                          If NI_DEVICE_TYPE_ENCODER is specified, data that
 *                          was read will be placed into ni_packet_t struct
 *                          inside the p_data pointer
 *                          If NI_DEVICE_TYPE_AI is specified, data that was
 *                          read will be placed into ni_frame_t struct inside
 *                          the p_data pointer
 *  \return On success
 *                          Total number of bytes read
 *          On failure
 *                          NI_RETCODE_INVALID_PARAM
 *                          NI_RETCODE_ERROR_NVME_CMD_FAILED
 *                          NI_RETCODE_ERROR_INVALID_SESSION
 ******************************************************************************/
LIB_API int ni_quadra_device_session_read(void *p_ctx, void *p_data,
                                                 int device_type);

/*!******************************************************************************
 *  \brief  Send a p_config command to configure scaling drawbox parameters.
 *
 *  \param[in/out] nifilter Pointer ni_quadra_filter_t
 *  \param[in] p_params     Pointer ni_quadra_scaler_drawbox_params_t
 *
 *  \return - NI_RETCODE_SUCCESS on success, NI_RETCODE_ERROR_INVALID_SESSION, NI_RETCODE_ERROR_NVME_CMD_FAILED on failure
*******************************************************************************/
LIB_API int ni_quadra_scaler_set_drawbox_params(
    ni_quadra_filter_t *nifilter, ni_quadra_scaler_drawbox_params_t *p_params);

/*!******************************************************************************
 *  \brief  Send a p_config command to configure watermark parameters.
 *
 *  \param[in/out] nifilter Pointer ni_quadra_filter_t
 *  \param[in] p_params     Pointer ni_quadra_scaler_watermark_params_t
 *
 *  \return - NI_RETCODE_SUCCESS on success, NI_RETCODE_ERROR_INVALID_SESSION, NI_RETCODE_ERROR_NVME_CMD_FAILED on failure
*******************************************************************************/
LIB_API int ni_quadra_scaler_set_watermark_params(
    ni_quadra_filter_t *nifilter, ni_quadra_scaler_watermark_params_t *p_params);

/*!*****************************************************************************
*  \brief  Get data from device
*
*  \param[in/out] nifilter Pointer ni_quadra_filter_t
*  \param[in] data    frame data
*  \param[in] width   frame width
*  \param[in] height  frame height
*
*  \return 0
*
******************************************************************************/
LIB_API int ni_quadra_filter_get_output_data(ni_quadra_filter_t *nifilter,
                                             uint8_t *data, int width,
                                             int height);

/*!*****************************************************************************
*  \brief  Get niframe data3 from device
*
*  \param[in/out] p_frame Pointer void
*
*  \return Pointer to uint8_t
*
******************************************************************************/
uint8_t *ni_quadra_filter_get_data3(void *p_frame);

/*!*****************************************************************************
*  \brief  prepare frames for ni_drawtext filter
*
*  \param[in/out] niframe Pointer void
*
*  \return 0
*
******************************************************************************/
LIB_API int ni_quadra_filter_drawtext_prep_frames(void **niframe);

/*!*****************************************************************************
 *  \brief  Allocate preliminary memory for the frame buffer based on provided
 *          parameters.
 *
 *  \param[in] niframe       Pointer void
 *  \param[in] video_width   Width of the video frame
 *  \param[in] video_height  Height of the video frame
 *  \param[in] alignment     Allignment requirement
 *  \param[in] pixel_format  Format for input
 *
 *  \return On success
 *                          NI_RETCODE_SUCCESS
 *          On failure
 *                          NI_RETCODE_INVALID_PARAM
 *                          NI_RETCODE_ERROR_MEM_ALOC
 ******************************************************************************/
LIB_API int ni_quadra_filter_frame_buffer_alloc_dl(void *niframe,
                                                   int video_width,
                                                   int video_height,
                                                   int pixel_format);

/*!*****************************************************************************
*  \brief  Close filter
*
*  \param[in/out] nifilter Pointer ni_quadra_filter_t
*
*  \return none
*
******************************************************************************/
LIB_API void ni_quadra_filter_close(ni_quadra_filter_t *nifilter);

/*!*****************************************************************************
 *  \brief   configure a network context based with the network binary
 *
 *  \param[in]  nifilter        Pointer to ni_quadra_filter_t
 *  \param[in]  p_network       Pointer to caller allocated ni_quadra_network_data_t
 *  \param[in]  file            Pointer to caller network binary file path
 *
 *  \return     NI_RETCODE_SUCCESS
 *              NI_RETCODE_INVALID_PARAM
 *              NI_RETCODE_ERROR_MEM_ALOC
 *              NI_RETCODE_ERROR_INVALID_SESSION
 *              NI_RETCODE_ERROR_NVME_CMD_FAILED
 *              NI_RETCODE_FAILURE
 *
 ******************************************************************************/
LIB_API int
ni_quadra_ai_config_network_binary(ni_quadra_filter_t *nifilter,
                                   ni_quadra_network_data_t *p_network,
                                   const char *file);

/*!*****************************************************************************
 *  \brief   configure a hvsplus context based with the network binary
 *
 *  \param[in]  nifilter        Pointer to ni_quadra_filter_t
 *  \param[in]  p_network       Pointer to caller allocated ni_quadra_network_data_t
 *  \param[in]  nb_width        Width for network binary
 *  \param[in]  nb_height       Height for network binary
 *  \param[in]  level           Hvsplus level
 *  \param[in]  format          Pixel format
 *
 *  \return     NI_RETCODE_SUCCESS
 *              NI_RETCODE_INVALID_PARAM
 *              NI_RETCODE_ERROR_MEM_ALOC
 *              NI_RETCODE_ERROR_INVALID_SESSION
 *              NI_RETCODE_ERROR_NVME_CMD_FAILED
 *              NI_RETCODE_FAILURE
 *
 ******************************************************************************/
LIB_API int
ni_quadra_ai_config_hvsplus(ni_quadra_filter_t *nifilter,
                                ni_quadra_network_data_t *p_network,
                                int nb_width,
                                int nb_height,
                                int level,
                                int format);

/*!*****************************************************************************
 *  \brief   Allocate input layers memory for AI frame buffer based on provided parameters
 *           taking into account width, height, format defined by network.
 *
 *  \param[out] p_frame        Pointer to void *
 *  \param[in]  p_network       Pointer to caller allocated ni_network_data_t
 *
 *  \return     NI_RETCODE_SUCCESS
 *              NI_RETCODE_INVALID_PARAM
 *              NI_RETCODE_ERROR_MEM_ALOC
 *
 ******************************************************************************/
LIB_API int
ni_quadra_ai_frame_buffer_alloc(void *p_frame,
                                ni_quadra_network_data_t *p_network);

/*!*****************************************************************************
 *  \brief  Allocate output layers memory for the packet buffer based on provided network
 *
 *  \param[out] p_pkt        Pointer to void *
 *  \param[in] p_network     Pointer to a caller allocated
 *                                   ni_quadra_network_data_t struct
 *
 *  \return On success
 *                          NI_RETCODE_SUCCESS
 *          On failure
 *                          NI_RETCODE_INVALID_PARAM
 *                          NI_RETCODE_ERROR_MEM_ALOC
 ******************************************************************************/
LIB_API int
ni_quadra_ai_packet_buffer_alloc(void *p_pkt,
                                 ni_quadra_network_data_t *p_network);

/*!*****************************************************************************
 *  \brief  Get dims from ni_quadra_network_layer_params_t
 *
 *  \param[out] p_param      Pointer to ni_quadra_network_layer_params_t
 *
 *  \return uint32_t
 ******************************************************************************/
LIB_API uint32_t
ni_quadra_ai_network_layer_dims(ni_quadra_network_layer_params_t *p_param);

/*!*****************************************************************************
 *  \brief  Convert data from nipacket to dst data
 *
 *  \param[in/out] p_data        Pointer to void *
 *                               ni_session_data_io_t struct
 *  \param[in] dst               Pointer to float
 *  \param[in] num               output_number
 *  \param[in] p_network         Pointer to ni_quadra_network_data_t
 *  \param[in] layer             int value
 *
 *  \return uint32_t
 ******************************************************************************/
LIB_API int ni_quadra_network_layer_convert_output(
    void *p_data, float *dst, uint32_t num,
    ni_quadra_network_data_t *p_network, uint32_t layer);

/*!*****************************************************************************
 *  \brief   Allocate memory for the frame buffer based on provided parameters
 *           taking into account the pixel format, width, height, stride,
 *           alignment, and extra data
 *  \param[in]  p_frame         Pointer to caller allocated void
 *  \param[in]  pixel_format    a pixel format in ni_pix_fmt_t enum
 *  \param[in]  video_width     width, in pixels
 *  \param[in]  video_height    height, in pixels
 *  \param[in]  linesize        horizontal stride
 *  \param[in]  alignment       apply a 16 pixel height alignment (T408 only)
 *  \param[in]  extra_len       meta data size
 *
 *  \return     NI_RETCODE_SUCCESS
 *              NI_RETCODE_INVALID_PARAM
 *              NI_RETCODE_ERROR_MEM_ALOC
 *
 ******************************************************************************/
int ni_quadra_frame_buffer_alloc_pixfmt(void *p_frame, int pixel_format,
                                        int video_width, int video_height,
                                        int linesize[], int alignment,
                                        int extra_len);

/*!*****************************************************************************
*  \brief  Reads YUV data from hw descriptor stored location on device
*
*  \param[in] p_ctx        Pointer to a caller allocated
*  \param[in] p_data       Pointer to a caller allocated
*                          ni_session_data_io_t struct which contains either a
*                          ni_frame_t data frame or ni_packet_t data packet to
*                          send
*  \param[in] hwdesc       HW descriptor to find frame in XCODER
*  \return On success
*                          Total number of bytes read
*          On failure
*                          NI_RETCODE_INVALID_PARAM
*                          NI_RETCODE_ERROR_NVME_CMD_FAILED
*                          NI_RETCODE_ERROR_INVALID_SESSION
*******************************************************************************/
int ni_quadra_device_session_hwdl(void *p_ctx, void *p_data,
                                  niquadraFrameSurface1_t *hwdesc);

/*!*****************************************************************************
*  \brief  Init filter session context only malloc session
*
*  \param[in/out] ni_ctx  Pointer NIFramesContext
*
*  \return On success
*                          NI_RETCODE_SUCCESS
*          On failure
*                          NI_RETCODE_FAILURE
*
******************************************************************************/
LIB_API int ni_quadra_init_internal_session(NIFramesContext *ni_ctx);

/*!*****************************************************************************
*  \brief  Init filter split context malloc session and set it default
*
*  \param[in/out] ni_ctx  Pointer NIFramesContext
*  \param[in] w           avctx width
*  \param[in] h           avctx height
*
*  \return On success
*                          NI_RETCODE_SUCCESS
*          On failure
*                          NI_RETCODE_FAILURE
*
******************************************************************************/
LIB_API int ni_quadra_init_split_rsrc(NIFramesContext *ni_ctx, int w, int h);

/*!*****************************************************************************
*  \brief  Init filter frame context and set value to session context
*
*  \param[in/out] ni_ctx      Pointer NIFramesContext
*  \param[in] width           avctx width
*  \param[in] height          avctx height
*  \param[in] ni_fmat         avctx format
*  \param[in] keep_alive_time keep_alive_time
*
*  \return On success
*                          NI_RETCODE_SUCCESS
*          On failure
*                          NI_RETCODE_FAILURE
*
******************************************************************************/
LIB_API int ni_quadra_filter_frames_init(NIFramesContext *ni_ctx, int *width,
                                         int *height, int ni_fmat,
                                         int keep_alive_time);

/*!*****************************************************************************
*  \brief  Close filter context and frame
*
*  \param[in/out] ni_ctx      Pointer NIFramesContext
*  \param[in] pool_size       avctx width
*
*  \return On success
*                             NI_RETCODE_SUCCESS
*          On failure
*                             NI_RETCODE_FAILURE
*
******************************************************************************/
LIB_API int ni_quadra_filter_frames_close(NIFramesContext *ni_ctx,
                                          int pool_size);

/*!*****************************************************************************
*  \brief  Close device
*
*  \param[in/out] ni_hw_ctx     Pointer AVNIDeviceContext
*
*  \return On success
*                               NI_RETCODE_SUCCESS
*          On failure
*                               NI_RETCODE_FAILURE
*
******************************************************************************/
LIB_API void ni_quadra_filter_device_close(AVNIDeviceContext *ni_hw_ctx);

/*!*****************************************************************************
*  \brief  Open hwupload filter
*
*  \param[in/out] ni_ctx     Pointer NIFramesContext
*
*  \return On success
*                            NI_RETCODE_SUCCESS
*          On failure
*                            NI_RETCODE_FAILURE
*
******************************************************************************/
LIB_API int ni_quadra_upload_device_session_open(NIFramesContext *ni_ctx);

/*!*****************************************************************************
*  \brief  Sends frame pool setup info to device
*
*  \param[in] ni_ctx       Pointer to NIFramesContext
*  \param[in] pool_size    Upload session initial allocated frames count
*                          must be > 0,
*  \param[in] pool         0 use the normal pool
*                          1 use a dedicated P2P pool
*
*  \return On success      Return code
*          On failure
*                          NI_RETCODE_FAILURE
*                          NI_RETCODE_ERROR_NVME_CMD_FAILED
*                          NI_RETCODE_ERROR_INVALID_SESSION
*                          NI_RETCODE_ERROR_MEM_ALOC
*******************************************************************************/
LIB_API int ni_quadra_filter_device_session_init_framepool(
    NIFramesContext *ni_ctx, uint32_t pool_size, uint32_t pool);

/*!*****************************************************************************
*  \brief  Sends frame pool setup info to device
*
*  \param[in] ni_hw_ctx    Pointer to AVNIDeviceContext
*  \param[in] device       string device name
*
*  \return On success
*                            NI_RETCODE_SUCCESS
*          On failure
*                            NI_RETCODE_FAILURE
*
*******************************************************************************/
LIB_API int ni_quadra_filter_device_create(AVNIDeviceContext *ni_hw_ctx,
                                           const char *device);

/*!*****************************************************************************
*  \brief  Sends frame pool setup info to device
*
*  \param[in/out] ni_ctx   Pointer to NIFramesContext
*  \param[in] dstbuf       Pointer dst buf
*
*  \param[in] dstdata      Pointer dst data
*  \param[in] buf_size     Pointer dst buf_size
*  \param[in] width        frame width
*  \param[in] height       frame height
*  \param[in] is_planar    frame format is nv12
*
*  \return On success
*                            NI_RETCODE_SUCCESS
*          On failure
*                            NI_RETCODE_ERROR_MEM_ALOC
*******************************************************************************/
LIB_API int ni_quadra_filter_get_buffer(NIFramesContext *ni_ctx,
                                        uint8_t **dstbuf, uint8_t **dstdata,
                                        uint32_t *buf_size, int width,
                                        int height, int is_planar);
/*!*****************************************************************************
*  \brief  Free filter frame
*
*  \param[in] opaque     Pointer to void*
*  \param[in] data       Pointer to void*
*
*  \return none
*
*******************************************************************************/
LIB_API void ni_quadra_filter_frame_free(void *opaque, uint8_t *data);

/*!*****************************************************************************
*  \brief  Upload frame
*
*  \param[in/out] ni_ctx   Pointer to NIFramesContext
*  \param[in] dst_surf     Pointer to niquadraFrameSurface1_t
*
*  \param[in] width        frame width
*  \param[in] height       frame height
*  \param[in] ni_fmat      frame format
*  \param[in] nb_planes    frame plane
*  \param[in] linesize     frame linesize
*  \param[in] src_data     pointer src data
*
*  \return On success
*                            NI_RETCODE_SUCCESS
*          On failure
*                            NI_RETCODE_FAILURE
*******************************************************************************/
LIB_API int ni_quadra_filter_hwup_frame(NIFramesContext *ni_ctx,
                                        niquadraFrameSurface1_t *dst_surf,
                                        int width, int height, int ni_fmat,
                                        int nb_planes, const int *linesize,
                                        uint8_t **src_data);

/*!*****************************************************************************
*  \brief  Download frame
*
*  \param[in/out] ni_ctx   Pointer to NIFramesContext
*  \param[in] src_surf     Pointer to niquadraFrameSurface1_t
*
*  \param[in] width        frame width
*  \param[in] height       frame height
*  \param[in] ni_fmat      frame format
*  \param[in] nb_planes    frame plane
*  \param[in] linesize     frame linesize
*  \param[in] dst_data     pointer dst data
*
*  \return On success
*                            NI_RETCODE_SUCCESS
*          On failure
*                            NI_RETCODE_FAILURE
*******************************************************************************/
LIB_API int ni_quadra_filter_hwdl_frame(NIFramesContext *ni_ctx,
                                        niquadraFrameSurface1_t *src_surf,
                                        int width, int height, int ni_fmat,
                                        int nb_planes, const int *linesize,
                                        uint8_t **dst_data);

/*!*****************************************************************************
*  \brief  Split context set
*
*  \param[in/out] ni_ctx   Pointer to NIFramesContext
*  \param[in] ni_surf      Pointer to niquadraFrameSurface1_t
*  \param[in] ni_surf1     Pointer to niquadraFrameSurface1_t
*  \param[in] ni_surf2     Pointer to niquadraFrameSurface1_t
*
*  \param[in] num_extra_outputs        split outputs number
*
*  \return none
*******************************************************************************/
LIB_API void ni_quadra_split_ctx_set(NIFramesContext *ni_ctx,
                                     niquadraFrameSurface1_t *ni_surf,
                                     niquadraFrameSurface1_t *ni_surf_1,
                                     niquadraFrameSurface1_t *ni_surf_2,
                                     int num_extra_outputs);

/*!*****************************************************************************
*  \brief  Get session device id
*
*  \param[in/out] session_ctx   Pointer to void*
*
*
*  \return hw id
*******************************************************************************/
LIB_API int ni_quadra_get_device_id(void *session_ctx);

/*!*****************************************************************************
*  \brief  Get session device session id
*
*  \param[in/out] session_ctx   Pointer to void*
*
*
*  \return session id
*******************************************************************************/
LIB_API uint32_t ni_quadra_get_session_id(void *session_ctx);

/*!*****************************************************************************
*  \brief  Close device handle
*
*  \param[in] handle        Handle to close
*
*
*******************************************************************************/
LIB_API void ni_quadra_device_close(int32_t handle);

/*!*****************************************************************************
*  \brief  Get session device handle id
*
*  \param[in/out] session_ctx   Pointer to void*
*
*
*  \return session handle id
*******************************************************************************/
LIB_API int32_t ni_quadra_get_device_handle(void *session_ctx);

/*!*****************************************************************************
*  \brief  Set session device handle
*
*  \param[in] session_ctx   Pointer to void*
*  \param[in] handle        Handle to set
*
*
*******************************************************************************/
LIB_API void ni_quadra_set_device_handle(void *session_ctx, int32_t handle);

/*!*****************************************************************************
*  \brief  Set download filter session context
*
*  \param[in/out] session_ctx   Pointer to void*
*  \param[in] value             Bool need to set value
*
*  \return none
*******************************************************************************/
LIB_API void ni_quadra_filter_session_dl_set(void *session_ctx, bool value);

/*!*****************************************************************************
*  \brief  Copy filter session context
*
*  \param[in/out] src_ctx   Pointer to void*
*  \param[in/out] dst_ctx   Pointer to void*
*
*  \return On success
*                            NI_RETCODE_SUCCESS
*          On failure
*                            NI_RETCODE_FAILURE
*******************************************************************************/
LIB_API int ni_quadra_device_session_copy(void *src_ctx, void *dst_ctx);

/*!*****************************************************************************
*  \brief  Copy filter session context
*
*  \param[in/out] src_ctx   Pointer to void*
*  \param[in/out] dst_ctx   Pointer to void*
*
*  \return On success
*                            NI_RETCODE_SUCCESS
*          On failure
*                            NI_RETCODE_FAILURE
*******************************************************************************/
LIB_API void ni_quadra_session_timestamp_copy(void *src_ctx, void *dst_ctx);

/*!*****************************************************************************
*  \brief  Get session context run status
*
*  \param[in/out] session_ctx   Pointer to void*
*
*
*  \return session context run status
*******************************************************************************/
LIB_API int ni_quadra_get_session_runstatus(void *session_ctx);

/*!*****************************************************************************
*  \brief  Set session context run status
*
*  \param[in/out] session_ctx   Pointer to void*
*
*
*  \return none
*******************************************************************************/
LIB_API void ni_quadra_set_session_runstatus(void *session_ctx, int status);

/*!*****************************************************************************
*  \brief  Get session context status
*
*  \param[in/out] session_ctx   Pointer to void*
*
*
*  \return session context status
*******************************************************************************/
LIB_API int ni_quadra_get_session_status(void *session_ctx);

/*!*****************************************************************************
*  \brief  Get session context blk_io_handle
*
*  \param[in/out] session_ctx   Pointer to void*
*
*
*  \return session context blk_io_handle
*******************************************************************************/
LIB_API int ni_quadra_get_session_blkiohandle(void *session_ctx);

/*!*****************************************************************************
*  \brief  Get niframui16FrameIdxe 
*
*  \param[in/out] niframe   Pointer to void*
*
*
*  \return frame ui16FrameIdx
*******************************************************************************/
LIB_API uint16_t ni_quadra_get_frame_idx(void *niframe);

/*!*****************************************************************************
*  \brief  Get niframe address
*
*  \param[in/out] niframe   Pointer to void*
*
*
*  \return point niframe
*******************************************************************************/
LIB_API uint8_t *ni_quadra_get_niframe(void *niframe);

/*!*****************************************************************************
*  \brief  Get niframe buffer address
*
*  \param[in/out] niframe   Pointer to void*
*
*
*  \return point frame p_buffer
*******************************************************************************/
LIB_API uint8_t *ni_quadra_get_frame_buffer(void *niframe);

/*!*****************************************************************************
*  \brief  Get niframe buffer size
*
*  \param[in/out] niframe   Pointer to void*
*
*
*  \return niframe buffer size
*******************************************************************************/
LIB_API uint32_t ni_quadra_get_frame_buffer_size(void *niframe);

/*!*****************************************************************************
*  \brief  Get niframe data address
*
*  \param[in/out] niframe   Pointer to void*
*
*
*  \return niframe data address
*******************************************************************************/
LIB_API uint8_t **ni_quadra_get_frame_data(void *niframe);

/*!*****************************************************************************
*  \brief  Get nipacket data address
*
*  \param[in/out] nipacket   Pointer to void*
*
*
*  \return nipacket data address
*******************************************************************************/
LIB_API uint8_t *ni_quadra_get_packet_data(void *nipacket);

/*!*****************************************************************************
*  \brief  Get nipacket data len
*
*  \param[in/out] nipacket   Pointer to void*
*
*
*  \return nipacket data len
*******************************************************************************/
LIB_API uint32_t ni_quadra_get_packet_data_len(void *nipacket);

/*!*****************************************************************************
*  \brief  Get niframe data datalen
*
*  \param[in/out] niframe   Pointer to void*
*  \param[in] dst_datalen   Pointer to int*
*
*
*  \return void
*******************************************************************************/
LIB_API void ni_quadra_get_frame_data_len(void *niframe, uint32_t *dst_datalen);

/*!*****************************************************************************
*  \brief  Recycle a frame buffer on card
*
*  \param[in] session_ctx   Pointer to void*
*  \param[in] niframe       Pointer to void*
*
*  \return On success    NI_RETCODE_SUCCESS
*          On failure    NI_RETCODE_INVALID_PARAM
*******************************************************************************/
LIB_API int ni_quadra_filter_hwframe_recycle(void *session_ctx, void *niframe);

/*!*****************************************************************************
 *  \brief  Configure the 2D engine to work based on provided parameters
 *
 *  \param[in]  p_ctx       pointer to session context
 *  \param[in]  p_cfg       pointer to ni_quadra_frame_config_t
 *
 *  \return         NI_RETCODE_INVALID_PARAM
 *                  NI_RETCODE_ERROR_INVALID_SESSION
 *                  NI_RETCODE_ERROR_NVME_CMD_FAILED
 *                  NI_RETCODE_ERROR_MEM_ALOC
 ******************************************************************************/
LIB_API int
ni_quadra_filter_device_config_frame(void *p_ctx,
                                     ni_quadra_frame_config_t *p_cfg);

/*!*****************************************************************************
*  \brief  Recycle a frame buffer on card
*
*  \param[in] surface   Struct containing device and frame location to clear out
*  \param[in] device_handle  handle to access device memory buffer is stored in
*
*  \return On success    NI_RETCODE_SUCCESS
*          On failure    NI_RETCODE_INVALID_PARAM
*******************************************************************************/
LIB_API int ni_quadra_hwframe_buffer_recycle(niquadraFrameSurface1_t *surface,
                                             int32_t device_handle);

/*!*****************************************************************************
 *  \brief  Get dimension information of Netint HW YUV420p frame to be sent
 *          to encoder for encoding. Caller usually retrieves this info and
 *          uses it in the call to ni_encoder_frame_buffer_alloc for buffer
 *          allocation.
 *
 *  \param[in]  width   source YUV frame width
 *  \param[in]  height  source YUV frame height
 *  \param[in]  factor  1 for 8 bit, 2 for 10 bit
 *  \param[in]  is_semiplanar  1 for semiplanar frame, 0 otherwise
 *  \param[out] plane_stride  size (in bytes) of each plane width
 *  \param[out] plane_height  size of each plane height
 *
 *  \return Y/Cb/Cr stride and height info
 *
 ******************************************************************************/
LIB_API void ni_quadra_get_hw_yuv420p_dim(int width, int height, int factor,
                                          int is_semiplanar,
                                          int plane_stride[4],
                                          int plane_height[4]);

/*!*****************************************************************************
*  \brief  Recycle a frame buffer on card
*
*  \param[in] surface   Struct containing device and frame location to clear out
*  \param[in] device_handle  handle to access device memory buffer is stored in
*
*  \return On success    NI_RETCODE_SUCCESS
*          On failure    NI_RETCODE_INVALID_PARAM
*******************************************************************************/
LIB_API int ni_quadra_get_seiset_size();

/*!******************************************************************************
 *  \brief Initialize a fifo buffer
 *
 *  \param[in] number_of_buffers  Total number of buffers in this fifo
 *  \param[in] size               The size of every buffer
 *
 *  \return On success return fifo pointer, On failure return NULL
 *******************************************************************************/
LIB_API ni_fifo_buffer_t *ni_fifo_initialize(uint32_t number_of_buffers,
                                             uint32_t size);

/*!******************************************************************************
 *  \brief Free a fifo
 *
 *  \param[in] p_fifo  The pointer of ni_fifo_buffer_t
 *
 *  \return
 *******************************************************************************/
LIB_API void ni_fifo_free(ni_fifo_buffer_t *p_fifo);

/*!******************************************************************************
 *  \brief Check if a fifo is full
 *
 *  \param[in] p_fifo  The pointer of ni_fifo_buffer_t
 *
 *  \return 1 is full; 0 is not full
 *******************************************************************************/
LIB_API int ni_fifo_is_full(ni_fifo_buffer_t *p_fifo);

/*!******************************************************************************
 *  \brief Check if a fifo is empty
 *
 *  \param[in] p_fifo  The pointer of ni_fifo_buffer_t
 *
 *  \return 1 is empty; 0 is not empty
 *******************************************************************************/
LIB_API int ni_fifo_is_empty(ni_fifo_buffer_t *p_fifo);

/*!******************************************************************************
 *  \brief Get number of buffers used in fifo
 *
 *  \param[in] p_fifo  The pointer of ni_fifo_buffer_t
 *
 *  \return fifo's number_of_buffers_used
 *******************************************************************************/
LIB_API uint32_t ni_fifo_number_of_buffers(ni_fifo_buffer_t *p_fifo);

/*!******************************************************************************
 *  \brief Get free buffer to write in the fifo
 *
 *  \param[in] p_fifo  The pointer of ni_fifo_buffer_t
 *
 *  \return On success return the pointer of free buffer,
 *          On failure return NULL
 *******************************************************************************/
LIB_API uint8_t *ni_fifo_generic_write(ni_fifo_buffer_t *p_fifo);
/*!******************************************************************************
 *  \brief Get first filled buffer to read in the fifo
 *
 *  \param[in] p_fifo  The pointer of ni_fifo_buffer_t
 *
 *  \return On success return the pointer of filled buffer in fifo,
 *          On failure return NULL
 *******************************************************************************/
LIB_API uint8_t *ni_fifo_generic_read(ni_fifo_buffer_t *p_fifo);

/*!******************************************************************************
 *  \brief Push back the last read buffer to the fifo
 *
 *  \param[in] p_fifo  The pointer of ni_fifo_buffer_t
 *
 *  \return On success return 0,
 *          On failure return -1
 *******************************************************************************/
LIB_API int ni_fifo_return_read(ni_fifo_buffer_t *p_fifo);

/*!*****************************************************************************
 *  \brief  Find the next start code
 *
 *  \param[in]  p pointer to buffer start address.
 *  \param[in]  end pointer to buffer end address.
 *  \param[state]  state pointer to nalu type address
 *
 *  \return search end address
 ******************************************************************************/
LIB_API const uint8_t *ni_find_start_code(const uint8_t *p, const uint8_t *end,
                                          uint32_t *state);

/*!*****************************************************************************
 *  \brief  Set ni_log_level
 *
 *  \param  level log level
 *
 *  \return
 ******************************************************************************/
LIB_API void ni_quadra_log_set_level(int level);

/*!*****************************************************************************
 *  \brief  Set ni_log() callback
 *
 *  \param[in] callback
 *
 *  \return
 ******************************************************************************/
LIB_API void ni_quadra_log_set_callback(void (*log_callback)(int, const char *,
                                                             va_list));

/*!*****************************************************************************
 *  \brief Convert ffmpeg log level integer to appropriate ni_quadra_log_level_t
 *
 *  \param fflog_level integer representation of FFmpeg log level
 *
 *  \return ni_quadra_log_level
 ******************************************************************************/
LIB_API ni_quadra_log_level_t ni_quadra_ff_to_ni_log_level(int fflog_level);

/*!******************************************************************************
 *  \brief  Create a pointer to  hw_device_info_coder_param_t instance. This instance will be created and 
 *          set to default vaule by param mode.You may change the resolution, fps, bit_8_10 and other vaule you want to set.
 *  
 *  \param[in] mode:0:create instance with decoder_param ,encoder_param, scaler_param and ai_param will be set to NULL
 *                  1:create instance with encoder_param ,decoder_param, scaler_param and ai_param will be set to NULL
 *                  2:create instance with scaler_param ,decoder_param, encoder_param and ai_param will be set to NULL
 *                  3:create instance with ai_param ,decoder_param, encoder_param and scaler_param will be set to NULL
 *                  >= 4:create instance with decoder_param encoder_param, scaler_param and ai_param for ni_check_hw_info() hw_mode
 * 
 *  \return  NULL-error,pointer to an instance when success
 *******************************************************************************/
LIB_API ni_quadra_hw_device_info_coder_param_t *ni_quadra_create_hw_device_info_coder_param(int mode);

/*!******************************************************************************
 *  \brief  Release a pointer to  hw_device_info_coder_param_t instance created by create_hw_device_info_coder_param
 *
 *  
 *  \param[in] p_hw_device_info_coder_param:pointer to a hw_device_info_coder_param_t instance created by create_hw_device_info_coder_param
 *                     
 *******************************************************************************/
LIB_API void ni_quadra_destory_hw_device_info_coder_param(ni_quadra_hw_device_info_coder_param_t *p_quadra_hw_device_info_coder_param);

/*!******************************************************************************
 *  \brief  Create a pointer to ni_hw_device_info_quadra_t instance .
 *  
 *  \param[in] device_type_num:number of device type to be allocated in this function
 *                 
 *  \param[in] avaliable_card_num:number of avaliable card per device to be allocated in this function
 * 
 *  \return  NULL-error,pointer to an instance when success
 *******************************************************************************/
LIB_API ni_quadra_hw_device_info_t *ni_quadra_hw_device_info_alloc(int device_type_num,int avaliable_card_num);

/*!******************************************************************************
 *  \brief  Release a pointer to  ni_hw_device_info_quadra_t instance created by create_hw_device_info_coder_param
 *
 *  
 *  \param[in] p_hw_device_info:pointer to a ni_hw_device_info_quadra_t instance created by create_hw_device_info_coder_param
 *                     
 *******************************************************************************/
LIB_API void ni_quadra_hw_device_info_free(ni_quadra_hw_device_info_t *p_hw_device_info);

/*!*****************************************************************************
 *  \brief  check hw info, return the appropriate card number to use depends on the load&task_num&used resource
 *
 *  \param[out] pointer_to_p_hw_device_info : pointer to user-supplied ni_hw_device_info_quadra_t (allocated by ni_hw_device_info_alloc).
 *                                            May be a ponter to NULL ,in which case a ni_hw_device_info_quadra_coder_param_t is allocated by this function
 *                                            and written to pointer_to_p_hw_device_info.
 *                                            record the device info, including  available card num and which card to select,
 *                                            and each card's informaton, such as, the load, task num, device type
 *  \param[in] task_mode: affect the scheduling strategy,
 *                        1 - both the load_num and task_num should consider, usually applied to live scenes
 *                        0 - only consider the task_num, don not care the load_num
 *  \param[in] hw_info_threshold_param : an array of threshold including device type task threshold and load threshold
 *                                       in hw_mode fill the arry with both encoder and decoder threshold or 
 *                                       fill the arry with preferential device type threshold when don not in hw_mode
 *                                       load threshold in range[0:100] task num threshold in range [0:32]
 *  \param[in] preferential_device_type : which device type is preferential 0:decode 1:encode .
 *                                        This need to set to encoder/decoder even if in sw_mode to check whether coder_param is wrong.
 *  \param[in] coder_param : encoder and decoder information that helps to choose card .This coder_param can be created and 
 *                           set to default value by function hw_device_info_coder_param_t * create_hw_device_info_coder_param().
 *                           You may change the resolution fps bit_8_10 or other vaule you want to use
 *  \param[in] hw_mode:Set 1 then this function will choose encoder and decoder in just one card .
 *                     When no card meets the conditions ,NO card will be choosed.
 *                     You can try to use set hw_mode 0 to use sw_mode to do encoder/decoder in different card when hw_mode reports an error
 *                     In hw_mode set both encoder_param and decoder_param in coder_param.
 *                     Set 0 then just consider sw_mode to choose which card to do encode/decode,
 *                     In sw_mode set one param in coder_param the other one will be set to NULL.
 *  \param[in] consider_mem : set 1 this function will consider memory usage extra
 *                            set 0 this function will not consider memory usage
 *
 *  \return  0-error 1-success
 *******************************************************************************/
LIB_API int ni_quadra_check_hw_info(ni_quadra_hw_device_info_t **pointer_to_p_hw_device_info, 
                             int task_mode,
                             ni_quadra_hw_device_info_threshold_param_t *hw_info_threshold_param,
                             int preferential_device_type,
                             ni_quadra_hw_device_info_coder_param_t * coder_param,
                             int hw_mode,
                             int consider_mem);

/*!****************************************************************************
* \brief      Get GUID of the device by block device name and type
*
* \param[in]  blk_name   device's block name
* \param[in]  type       device type
*
* \return     device GUID (>= 0) if found, NI_RETCODE_FAILURE (-1) otherwise
*******************************************************************************/
LIB_API int ni_quadra_rsrc_get_device_by_block_name(const char *blk_name,
                                                    ni_quadra_device_type_t device_type);

/*!*****************************************************************************
 *  \brief  Open a new device session depending on the device_type parameter
 *          If device_type is NI_DEVICE_TYPE_DECODER opens decoding session
 *          If device_type is NI_DEVICE_TYPE_ENCODER opens encoding session
 *          If device_type is NI_DEVICE_TYPE_SCALER opens scaling session
 *
 *  \param[in] p_ctx        Pointer to a caller allocated
 *                          ni_session_context_t struct
 *  \param[in] device_type  NI_DEVICE_TYPE_DECODER, NI_DEVICE_TYPE_ENCODER,
 *                          or NI_DEVICE_TYPE_SCALER
 *  \return On success
 *                          NI_RETCODE_SUCCESS
 *          On failure
 *                          NI_RETCODE_INVALID_PARAM
 *                          NI_RETCODE_ERROR_MEM_ALOC
 *                          NI_RETCODE_ERROR_NVME_CMD_FAILED
 *                          NI_RETCODE_ERROR_INVALID_SESSION
 ******************************************************************************/
LIB_API int ni_quadra_device_session_open(void *p_ctx,
                                    ni_quadra_device_type_t device_type);

/*!*****************************************************************************
 *  \brief  Sends data to the device
 *          If device_type is NI_DEVICE_TYPE_DECODER sends data packet to
 *          decoder
 *          If device_type is NI_DEVICE_TYPE_ENCODER sends data frame to encoder
 *          If device_type is NI_DEVICE_TYPE_AI sends data frame to ai engine
 *
 *  \param[in] p_ctx        Pointer to a caller allocated
 *                          ni_session_context_t struct
 *  \param[in] p_data       Pointer to a caller allocated
 *                          ni_session_data_io_t struct which contains either a
 *                          ni_frame_t data frame or ni_packet_t data packet to
 *                          send
 *  \param[in] device_type  NI_DEVICE_TYPE_DECODER or NI_DEVICE_TYPE_ENCODER or
 *                          NI_DEVICE_TYPE_AI
 *                          If NI_DEVICE_TYPE_DECODER is specified, it is
 *                          expected that the ni_packet_t struct inside the
 *                          p_data pointer contains data to send.
 *                          If NI_DEVICE_TYPE_ENCODER or NI_DEVICE_TYPE_AI is
 *                          specified, it is expected that the ni_frame_t
 *                          struct inside the p_data pointer contains data to
 *                          send.
 *  \return On success
 *                          Total number of bytes written
 *          On failure
 *                          NI_RETCODE_INVALID_PARAM
 *                          NI_RETCODE_ERROR_NVME_CMD_FAILED
 *                          NI_RETCODE_ERROR_INVALID_SESSION
 ******************************************************************************/
LIB_API int ni_quadra_device_session_write(void *p_ctx, void *p_data,
                                   ni_quadra_device_type_t device_type);

/*!*****************************************************************************
 *  \brief  Free packet buffer that was previously allocated with
 *          ni_packet_buffer_alloc
 *
 *  \param[in] p_data    Pointer to void *
 *                       ni_session_data_io_t struct
 *
 *  \return On success    NI_RETCODE_SUCCESS
 *          On failure    NI_RETCODE_INVALID_PARAM
 ******************************************************************************/
LIB_API int ni_quadra_packet_buffer_free(void* p_data);

/*!*****************************************************************************
 *  \brief  Free frame buffer that was previously allocated with either
 *          ni_frame_buffer_alloc or ni_encoder_frame_buffer_alloc or
 *          ni_frame_buffer_alloc_nv
 *
 *  \param[in] p_data    Pointer to void *
 *                       ni_session_data_io_t struct
 *
 *  \return On success    NI_RETCODE_SUCCESS
 *          On failure    NI_RETCODE_INVALID_PARAM
 ******************************************************************************/
LIB_API int ni_quadra_frame_buffer_free(void* p_data);

/*!*****************************************************************************
 *  \brief  Create and return the allocated ni_device_pool_t struct
 *
 *  \param  None
 *
 *  \return Pointer to ni_device_pool_t struct on success, or NULL on failure
 *******************************************************************************/
LIB_API ni_quadra_device_pool_t* ni_quadra_rsrc_get_device_pool(void);

/*!*****************************************************************************
 *  \brief  Close device session that was previously opened by calling
 *          ni_device_session_open()
 *          If device_type is NI_DEVICE_TYPE_DECODER closes decoding session
 *          If device_type is NI_DEVICE_TYPE_ENCODER closes encoding session
 *          If device_type is NI_DEVICE_TYPE_SCALER closes scaling session
 *
 *  \param[in] p_ctx        Pointer to a caller allocated
 *                          ni_session_context_t struct
 *  \param[in] eos_received Flag indicating if End Of Stream indicator was
 *                          received
 *  \param[in] device_type  NI_DEVICE_TYPE_DECODER, NI_DEVICE_TYPE_ENCODER,
 *                          or NI_DEVICE_TYPE_SCALER
 *  \return On success
 *                          NI_RETCODE_SUCCESS
 *          On failure
 *                          NI_RETCODE_INVALID_PARAM
 *                          NI_RETCODE_ERROR_NVME_CMD_FAILED
 *                          NI_RETCODE_ERROR_INVALID_SESSION
 ******************************************************************************/
LIB_API int ni_quadra_device_session_close(void *p_ctx,
                                            int eos_recieved,
                                            ni_quadra_device_type_t device_type);

/*!*****************************************************************************
 *  \brief  Clear already allocated session context
 *
 *  \param[in]  p_ctx Pointer to an already allocated ni_session_context_t
 *
 *
 ******************************************************************************/
LIB_API void ni_quadra_device_session_context_clear(void *p_ctx);

LIB_API int ni_quadra_frame_buffer_alloc_hwenc(void *niframe,
                                        int video_width, int video_height,
                                        int extra_len);

LIB_API void ni_quadra_set_framerate(void *p_ctx, int num, int den);

/*!*****************************************************************************
 *  \brief  Get session_ctx send/recive packet number
 *
 *  \param[in]  p_ctx Pointer to an already allocated ni_session_context_t
 *
 *
 ******************************************************************************/
LIB_API int ni_quadra_get_session_packet_num(void *session_ctx);

/*!*****************************************************************************
 *  \brief  Get session_ctx send/recieve frame number
 *
 *  \param[in]  p_ctx Pointer to an already allocated ni_session_context_t
 *
 *
 ******************************************************************************/
LIB_API int ni_quadra_get_session_frame_num(void *session_ctx);

#ifdef __cplusplus
}
#endif
